/*
  main.cpp - Main loop for Arduino sketches
  Copyright (c) 2005-2013 Arduino Team.  All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <Arduino.h>
#include "EOS_Scheduler.h"
#include "Mutex.h"

//Declared weak in Arduino.h to allow user redefinitions.
int atexit(void (*func)()) { return 0; }

// Weak empty variant initialization function.
// May be redefined by variant files.
void initVariant() __attribute__((weak));
void initVariant() { }

uint16 __EOS_nTimerElapsed = 0;
static CMutex LockTimerElapsed;

int main(void)
{
	init();

	initVariant();

#if defined(USBCON)
	USBDevice.attach();
#endif
	
	setup();
    EOS_Scheduler.runInitTask();

    for (;;)
    {
        LockTimerElapsed.lock();
        if (__EOS_nTimerElapsed > 0)
        {
            uint16 nTimer = __EOS_nTimerElapsed;
            __EOS_nTimerElapsed = 0;
            LockTimerElapsed.unlock();
            EOS_Scheduler.schedule(nTimer);
        }
        else
        {
            LockTimerElapsed.unlock();
        }
        EOS_Scheduler.runIdleTask();

		loop();
		if (serialEventRun) serialEventRun();
	}
        
	return 0;
}

