//!	EOS_Delete.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_Delete_h_
#define _EOS_Delete_h_


//--------------------------- include section -----------------------
#include <stdlib.h>


//--------------------------- class definition ----------------------
//! malloc and free resp. new and delete are unwanted function calls
//! in embedded software development. When an EOS class needs to be
//! deleted the class needs to be derived from the class CEOS_Delete
//! \code
//! class CMyClass :
//!     public CEOS_New,
//!     public CEOS_Delete
//! {
//!     ...
//! };
//!
//! void function()
//! {
//!     // possible through CEOS_New:
//!     CMyClass* pMyClass = new CMyClass;
//!     // possible through CEOS_Delete:
//!     delete pMyClass;
//! }
//! \endcode
class CEOS_Delete
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_Delete);


//--------------------------- public access -------------------------
public:
    // operator delete ----------------------------------------------
    //! \internal
    inline void operator delete(void* pObject)
    {
        free(pObject);
    }
};


#endif
