//!	EOS_InitTask.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_InitTask_h_
#define _EOS_InitTask_h_


//--------------------------- include section -----------------------
#include "EOS_Task.h"


//--------------------------- class definition ----------------------
//! A CEOS_InitTask represents a task, which is called only once
//! during the initialization phase of the program execution.
//! This class is usually only used by the EOS_Scheduler
class CEOS_InitTask :
        public CEOS_Task
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! handing over all information to the base class CEOS_Task
    CEOS_InitTask(void(*pFunc)()) :
        CEOS_Task(pFunc)
    {
        // Nothing to do
    }

    // constructor --------------------------------------------------
    //! \internal
    //! handing over all information to the base class CEOS_Task
    template <class T_OBJECT>
    CEOS_InitTask(T_OBJECT* pObject,
                  void(T_OBJECT::* pFunc)()) :
        CEOS_Task(pObject,pFunc)
    {
        // Nothing to do
    }

    NO_COPY_AND_ASSIGNMENT(CEOS_InitTask);


//--------------------------- public access -------------------------
public:
    // isFinished ---------------------------------------------------
    //! a CEOS_InitTask is always finished after execution
    inline bool isFinished() const
    {
        return true;
    }
};

#endif
