//!	EOS_PWMOutput.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_PWMOutput_h_
#define _EOS_PWMOutput_h_


//--------------------------- include section -----------------------
#include "types.h"
#include "EOS_AnalogOutput.h"


//--------------------------- class definition ----------------------
//! this class represents an abstraction of an analog output capable
//! to perform PWM
//! \code
//! class CPin :
//!     public CEOS_PWMOutput
//! \endcode
class CEOS_PWMOutput : public CEOS_AnalogOutput
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    CONSTRUCTOR_EMPTY_NO_COPY_AND_ASSIGNMENT(CEOS_PWMOutput);


//--------------------------- public virtual access -----------------
public:
    // setPWMFrequency ----------------------------------------------
    //! sets the SFR to have a PWM frequency between nFrequencyLow and
    //! nFrequencyHigh\n
    //! Due the frequency is depending on the prescaler, a valid
    //! frequency will be selected between nFrequencyLow and
    //! nFrequencyHigh\n
    //! This is a pure virtual function and needs to be implemented
    //! in the derived classes according the µC datasheet
    //! \param nFrequencyLow is the lowest allowed PWM frequency in Hz
    //! \param nFrequencyHigh is the lowest allowed PWM frequency in Hz
    //! \return selected frequency for this PWM or 0 for not selectable
    //! \code
    //! // set the PWM frequency somwhere between 100 and 200 Hz
    //! if (Pin3.setPWMFrequency(100,200) == 0)
    //! {
    //!     // not possible
    //! }
    //! \endcode
    virtual unsigned int setPWMFrequency(unsigned int nFrequencyLow,
                                         unsigned int nFrequencyHigh) = 0;
};


#endif
