//!	EOS_TaskHandler.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_TaskHandler_h_
#define _EOS_TaskHandler_h_


//--------------------------- include section -----------------------
#include "EOS_HandlerBase.h"
#include "EOS_New.h"


//--------------------------- class definition ----------------------
//! A CEOS_TaskHandler is an internally used class, storing the
//! pointer to a member function of a class, which shall be executed
//! when requested.
//! This class is a template class with an object type and no further
//! arguments
template <class T_OBJECT>
class CEOS_TaskHandler :
        public CEOS_HandlerBase<>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! storing the pointer to the object holding the member function
    //! and the pointer to the member function
    CEOS_TaskHandler(T_OBJECT* pObject,
                      void(T_OBJECT::* pFunc)()) :
        m_pObject(pObject),
        m_pFunc(pFunc)
    {
        // Nothing to do
    }


//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    //! \internal
    //! call the member function of the object
    inline virtual void onExecute()
    {
        (m_pObject->* m_pFunc)();
    }


//--------------------------- protected data ------------------------
protected:
    // m_pObject ----------------------------------------------------
    //! \internal
    //! pointer to the object, which contains the member function
    T_OBJECT* m_pObject;

    // m_pFunc ------------------------------------------------------
    //! \internal
    //! pointer to the member function
    void(T_OBJECT::* m_pFunc)();
};

//########################### specialized templates #################

//--------------------------- class definition ----------------------
//! This class is a template class with no object type and no further
//! arguments
template <>
class CEOS_TaskHandler<void> :
        public CEOS_HandlerBase<>,
        public CEOS_New
{
//--------------------------- construction --------------------------
public:
    // constructor --------------------------------------------------
    //! \internal
    //! storing the pointer to the object holding the member function
    //! and the pointer to the member function
    CEOS_TaskHandler(void(pFunc)()) :
        m_pFunc(pFunc)
    {
        // Nothing to do
    }


//--------------------------- virtual public access -----------------
public:
    // onExecute ----------------------------------------------------
    //! \internal
    //! call the member function of the object
    inline virtual void onExecute()
    {
        (m_pFunc)();
    }


//--------------------------- protected data ------------------------
protected:
    // m_pFunc ------------------------------------------------------
    //! \internal
    //! pointer to the member function
    void(*m_pFunc)();
};


#endif
