//!	EOS_State.h


//--------------------------- inclusion guards ----------------------
#ifndef _EOS_State_h_
#define _EOS_State_h_


//--------------------------- include section -----------------------
#include "EOS_Macros.h"
#include <stdio.h>
#include "EOS_Scheduler.h"
#include "EOS_List.h"
#include <stdlib.h>

#include "EOS_Statemachine.h"

template <class T_OBJECT> class CEOS_Statemachine;


//--------------------------- class definition ----------------------
template <class T_OBJECT>
class CEOS_State
{
//--------------------------- family --------------------------------
friend class CEOS_Statemachine<T_OBJECT>;

//--------------------------- construction --------------------------
public:
    // Constructor --------------------------------------------------
    inline CEOS_State(CEOS_Statemachine<T_OBJECT>* pEOS_Statemachine) :
        m_pEOS_Statemachine(pEOS_Statemachine)
    {
        // Nothing to do
    }

    // Destructor ---------------------------------------------------
    inline ~CEOS_State()
    {
        // Nothing to do
    }

    // lock copy and assignement ------------------------------------
private:
    CEOS_State(const CEOS_State&);
    const CEOS_State& operator = (const CEOS_State&);

//--------------------------- public types --------------------------
public:
    // TMemberFunction ----------------------------------------------
    typedef void (T_OBJECT::* TMemberFunction)();


//--------------------------- protected types -----------------------
protected:
    // SStateInfo ---------------------------------------------------
    typedef struct
    {
        CEOS_Event* pEOS_Event;
        TMemberFunction pMemberFunction;
        CEOS_State<T_OBJECT>* pNextState;
    } SStateInfo;


//--------------------------- public access -------------------------
public:
    // onEvent ------------------------------------------------------
    void onEvent(CEOS_Event* pEOS_Event,
                 TMemberFunction pMemberFunction,
                 CEOS_State<T_OBJECT>* pNextState)
    {
        SStateInfo* pStateInfo = (SStateInfo*)malloc(sizeof(SStateInfo));
        pStateInfo->pEOS_Event = pEOS_Event;
        pStateInfo->pMemberFunction = pMemberFunction;
        pStateInfo->pNextState = pNextState;

        m_StateInfoList.add(pStateInfo);
    }

    // onEvent ------------------------------------------------------
    inline void onEvent(CEOS_Event* pEOS_Event,
                        TMemberFunction pMemberFunction)
    {
        onEvent(pEOS_Event,pMemberFunction,NULL);
    }

    // onEvent ------------------------------------------------------
    inline void onEvent(CEOS_Event* pEOS_Event,
                        CEOS_State<T_OBJECT>* pNextState)
    {
        onEvent(pEOS_Event,NULL,pNextState);
    }


//--------------------------- protected access ----------------------
protected:
    // onEvent ------------------------------------------------------
    void onEvent(CEOS_Event* pEOS_Event)
    {
        // Search the event
        typename CEOS_List<SStateInfo*>::TPosition pPosition = m_StateInfoList.first();
        while (pPosition != NULL)
        {
            SStateInfo* pStateInfo = m_StateInfoList.get(pPosition);
            if (pStateInfo->pEOS_Event == pEOS_Event)
            {
                // set next state now, might be overwritten by
                // function called
                if (pStateInfo->pNextState != NULL)
                {
                    m_pEOS_Statemachine->setState(pStateInfo->pNextState);
                }
                if (pStateInfo->pMemberFunction != NULL)
                {
                    // This is the reported event, call the member function
                    T_OBJECT* pObject = m_pEOS_Statemachine->m_pObject;
                    (pObject->* pStateInfo->pMemberFunction)();
                }
            }
            pPosition = m_StateInfoList.next(pPosition);
        }
    }

//--------------------------- protected access ----------------------
protected:
    // m_pEOS_Statemachine ------------------------------------------
    CEOS_Statemachine<T_OBJECT>* m_pEOS_Statemachine;

    // m_StateInfoList -----------------------------------------------
    CEOS_List<SStateInfo*> m_StateInfoList;

};



#endif
